<?php

namespace Drupal\performance_analyzer\Controller;

use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\performance_analyzer\Service\PerformanceAnalyzerService;
use Drupal\performance_analyzer\Service\ReportGeneratorService;
use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Url;

/**
 * Controller for the Performance Analyzer report.
 */
class AnalyzerController extends ControllerBase {

  /**
   * The performance analyzer service.
   *
   * @var \Drupal\performance_analyzer\Service\PerformanceAnalyzerService
   */
  protected $analyzer;

  /**
   * The report generator service.
   *
   * @var \Drupal\performance_analyzer\Service\ReportGeneratorService
   */
  protected $reportGenerator;

  /**
   * The date formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface
   */
  protected $dateFormatter;

  /**
   * Constructs a new AnalyzerController object.
   *
   * @param \Drupal\performance_analyzer\Service\PerformanceAnalyzerService $analyzer
   *   The performance analyzer service.
   * @param \Drupal\performance_analyzer\Service\ReportGeneratorService $reportGenerator
   *   The report generator service.
   * @param \Drupal\Core\Datetime\DateFormatterInterface $dateFormatter
   *   The date formatter service.
   */
  public function __construct(
    PerformanceAnalyzerService $analyzer,
    ReportGeneratorService $reportGenerator,
    DateFormatterInterface $dateFormatter
  ) {
    $this->analyzer = $analyzer;
    $this->reportGenerator = $reportGenerator;
    $this->dateFormatter = $dateFormatter;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('performance_analyzer.analyzer'),
      $container->get('performance_analyzer.report_generator'),
      $container->get('date.formatter')
    );
  }

  /**
   * Generates the performance analysis report.
   *
   * @return array
   *   The render array for the report.
   */
  public function generateReport() {
    $build = [
      '#theme' => 'performance_analyzer_report',
      '#attached' => [
        'library' => ['performance_analyzer/report'],
      ],
    ];

    try {
      $report = $this->analyzer->analyze();
      $build['#report'] = $report;
      
      // Summary statistics
      $build['summary'] = [
        '#type' => 'container',
        '#attributes' => ['class' => ['performance-analyzer-summary']],
        'stats' => [
          '#theme' => 'performance_analyzer_summary',
          '#stats' => $this->buildSummaryStats($report),
        ],
      ];

      // Module details
      $build['modules'] = [
        '#type' => 'container',
        '#attributes' => ['class' => ['performance-analyzer-modules']],
      ];

      foreach ($report as $moduleName => $moduleData) {
        $build['modules'][$moduleName] = [
          '#type' => 'details',
          '#title' => $this->t('@module (@score%)', [
            '@module' => $moduleName,
            '@score' => $moduleData['performance_score'],
          ]),
          '#open' => $moduleData['performance_score'] < 85,
          'content' => [
            '#theme' => 'performance_analyzer_module',
            '#module' => $moduleData,
          ],
        ];
      }

      // Download buttons
      $build['actions'] = $this->buildActionButtons();

    } 
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Error generating performance report: @message', [
        '@message' => $e->getMessage(),
      ]));
    }

    return $build;
  }

  /**
   * Builds summary statistics from the report data.
   *
   * @param array $report
   *   The performance analysis report.
   *
   * @return array
   *   Summary statistics.
   */
  protected function buildSummaryStats($report) {
    $stats = [
      'total_modules' => count($report),
      'total_issues' => 0,
      'high_severity' => 0,
      'medium_severity' => 0,
      'low_severity' => 0,
      'average_score' => 0,
    ];

    foreach ($report as $moduleData) {
      $stats['total_issues'] += count($moduleData['issues']);
      $stats['average_score'] += $moduleData['performance_score'];

      foreach ($moduleData['issues'] as $issue) {
        switch ($issue['severity']) {
          case 'high':
            $stats['high_severity']++;
            break;
          case 'medium':
            $stats['medium_severity']++;
            break;
          case 'low':
            $stats['low_severity']++;
            break;
        }
      }
    }

    if ($stats['total_modules'] > 0) {
      $stats['average_score'] = round($stats['average_score'] / $stats['total_modules'], 2);
    }

    return $stats;
  }

  /**
   * Builds the action buttons for the report page.
   *
   * @return array
   *   Render array for action buttons.
   */
  protected function buildActionButtons() {
    return [
      '#type' => 'container',
      '#attributes' => ['class' => ['performance-analyzer-actions']],
      'download_json' => [
        '#type' => 'link',
        '#title' => $this->t('Download JSON Report'),
        '#url' => Url::fromRoute('performance_analyzer.download_report', ['format' => 'json']),
        '#attributes' => [
          'class' => ['button', 'button--primary'],
        ],
      ],
      'download_html' => [
        '#type' => 'link',
        '#title' => $this->t('Download HTML Report'),
        '#url' => Url::fromRoute('performance_analyzer.download_report', ['format' => 'html']),
        '#attributes' => [
          'class' => ['button'],
        ],
      ],
      'refresh' => [
        '#type' => 'link',
        '#title' => $this->t('Refresh Analysis'),
        '#url' => Url::fromRoute('performance_analyzer.report'),
        '#attributes' => [
          'class' => ['button', 'use-ajax'],
          'data-dialog-type' => 'modal',
        ],
      ],
    ];
  }

  /**
   * Downloads the report in the specified format.
   *
   * @param string $format
   *   The format to download (json or html).
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The response containing the downloaded report.
   */
  public function downloadReport($format) {
    try {
      return $this->reportGenerator->generateReport($format);
    } 
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Error generating report: @message', [
        '@message' => $e->getMessage(),
      ]));
      return $this->redirect('performance_analyzer.report');
    }
  }
}