<?php

namespace Drupal\performance_analyzer\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

class AnalyzerSettingsForm extends ConfigFormBase {
    /**
     * Default values for settings.
     */
    protected const DEFAULT_SETTINGS = [
        'custom_modules_path' => 'modules/custom',
        'severity_thresholds' => [
            'high' => 85,
            'medium' => 70,
        ],
        'analysis_options' => [
            'checks' => [
                'entity_loading' => TRUE,
                'caching' => TRUE,
                'query_performance' => TRUE,
                'render_arrays' => TRUE,
                'static_cache' => TRUE,
                'service_usage' => TRUE,
                'coding_standards' => TRUE,
            ],
        ],
        'report_options' => [
            'include_examples' => TRUE,
            'group_by_severity' => TRUE,
        ],
    ];

    protected function getEditableConfigNames() {
        return ['performance_analyzer.settings'];
    }

    public function getFormId() {
        return 'performance_analyzer_settings';
    }

    public function buildForm(array $form, FormStateInterface $form_state) {
        $config = $this->config('performance_analyzer.settings');

        $form['custom_modules_path'] = [
            '#type' => 'textfield',
            '#title' => $this->t('Custom Modules Path'),
            '#default_value' => $config->get('custom_modules_path') ?? self::DEFAULT_SETTINGS['custom_modules_path'],
            '#description' => $this->t('Relative path from Drupal root to custom modules directory.'),
            '#required' => TRUE,
        ];

        $form['analysis_options'] = [
            '#type' => 'details',
            '#title' => $this->t('Analysis Options'),
            '#open' => TRUE,
            '#tree' => TRUE,
        ];

        $form['analysis_options']['checks'] = [
            '#type' => 'checkboxes',
            '#title' => $this->t('Enable Checks'),
            '#options' => [
                'entity_loading' => $this->t('Entity Loading Patterns'),
                'caching' => $this->t('Caching Implementation'),
                'query_performance' => $this->t('Query Performance'),
                'render_arrays' => $this->t('Render Array Optimization'),
                'static_cache' => $this->t('Static Cache Usage'),
                'service_usage' => $this->t('Service Implementation'),
                'coding_standards' => $this->t('Coding Standards'),
            ],
            '#default_value' => $config->get('analysis_options.checks') ?? array_keys(self::DEFAULT_SETTINGS['analysis_options']['checks']),
            '#description' => $this->t('Select which aspects to analyze.'),
        ];

        $form['severity_thresholds'] = [
            '#type' => 'details',
            '#title' => $this->t('Severity Thresholds'),
            '#open' => TRUE,
            '#tree' => TRUE,
            '#description' => $this->t('Define score thresholds for severity levels. High severity threshold should be higher than medium severity threshold.'),
        ];

        $form['severity_thresholds']['high'] = [
            '#type' => 'number',
            '#title' => $this->t('High Severity Threshold'),
            '#default_value' => $config->get('severity_thresholds.high') ?? self::DEFAULT_SETTINGS['severity_thresholds']['high'],
            '#description' => $this->t('Issues will be marked as high severity when the score is below this value.'),
            '#min' => 0,
            '#max' => 100,
            '#step' => 1,
            '#required' => TRUE,
        ];

        $form['severity_thresholds']['medium'] = [
            '#type' => 'number',
            '#title' => $this->t('Medium Severity Threshold'),
            '#default_value' => $config->get('severity_thresholds.medium') ?? self::DEFAULT_SETTINGS['severity_thresholds']['medium'],
            '#description' => $this->t('Issues will be marked as medium severity when the score is below this value but above the high severity threshold.'),
            '#min' => 0,
            '#max' => 100,
            '#step' => 1,
            '#required' => TRUE,
        ];

        $form['report_options'] = [
            '#type' => 'details',
            '#title' => $this->t('Report Options'),
            '#open' => TRUE,
            '#tree' => TRUE,
        ];

        $form['report_options']['group_by_severity'] = [
            '#type' => 'checkbox',
            '#title' => $this->t('Group Issues by Severity'),
            '#default_value' => $config->get('report_options.group_by_severity') ?? self::DEFAULT_SETTINGS['report_options']['group_by_severity'],
            '#description' => $this->t('Group issues by severity level in the report.'),
        ];

        return parent::buildForm($form, $form_state);
    }

    public function validateForm(array &$form, FormStateInterface $form_state) {
        parent::validateForm($form, $form_state);

        // Validate custom modules path
        $path = $form_state->getValue('custom_modules_path');
        if (!is_dir(DRUPAL_ROOT . '/' . $path)) {
            $form_state->setErrorByName('custom_modules_path', 
                $this->t('Directory @path does not exist.', ['@path' => $path])
            );
        }

        // Get threshold values
        $thresholds = $form_state->getValue('severity_thresholds');
        $high = (int) $thresholds['high'];
        $medium = (int) $thresholds['medium'];

        // Validate threshold values
        if ($medium >= $high) {
            $form_state->setErrorByName('severity_thresholds][medium',
                $this->t('Medium severity threshold (@medium) must be lower than high severity threshold (@high).', [
                    '@medium' => $medium,
                    '@high' => $high,
                ])
            );
        }
    }

    public function submitForm(array &$form, FormStateInterface $form_state) {
        $config = $this->config('performance_analyzer.settings');
        
        $checks = $form_state->getValue(['analysis_options', 'checks']);
        $enabled_checks = is_array($checks) ? array_filter($checks) : self::DEFAULT_SETTINGS['analysis_options']['checks'];

        $config
            ->set('custom_modules_path', $form_state->getValue('custom_modules_path'))
            ->set('analysis_options.checks', $enabled_checks)
            ->set('severity_thresholds', $form_state->getValue('severity_thresholds'))
            ->set('report_options', $form_state->getValue('report_options'))
            ->save();

        parent::submitForm($form, $form_state);
    }
}