<?php

namespace Drupal\performance_analyzer\Service;

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Config\ConfigFactoryInterface;

class PerformanceAnalyzerService {
  protected $entityTypeManager;
  protected $database;
  protected $cache;
  protected $logger;
  protected $fileSystem;
  protected $config;
  protected $results = [];
  
  public function __construct(
    EntityTypeManagerInterface $entityTypeManager,
    Connection $database,
    CacheBackendInterface $cache,
    LoggerChannelFactoryInterface $loggerFactory,
    FileSystemInterface $fileSystem,
    ConfigFactoryInterface $configFactory
    ) {
      $this->entityTypeManager = $entityTypeManager;
      $this->database = $database;
      $this->cache = $cache;
      $this->logger = $loggerFactory->get('performance_analyzer');
      $this->fileSystem = $fileSystem;
      $this->config = $configFactory->get('performance_analyzer.settings');
    }
    
    public function analyze() {
      try {
        $appRoot = \Drupal::root();
        $modulesPath = $this->config->get('custom_modules_path') ?? 'modules/custom';
        $fullPath = $appRoot . '/' . $modulesPath;
        
        $this->logger->notice('Starting module analysis in: @path', ['@path' => $fullPath]);
        
        if (!is_dir($fullPath)) {
          throw new \RuntimeException("Custom modules directory not found at: $fullPath");
        }
        
        if (!is_readable($fullPath)) {
          throw new \RuntimeException("Custom modules directory is not readable at: $fullPath");
        }
        
        $modules = $this->findCustomModules($fullPath);
        
        if (empty($modules)) {
          $this->logger->warning('No analyzable modules found in directory: @path', [
            '@path' => $fullPath
          ]);
          throw new \RuntimeException("No analyzable modules found in directory: $fullPath");
        }
        
        foreach ($modules as $module) {
          $this->analyzeModule($module);
        }
        
        return $this->results;
      }
      catch (\Exception $e) {
        $this->logger->error('Error analyzing modules: @message', ['@message' => $e->getMessage()]);
        throw $e;
      }
    }
    
    protected function isModuleInstalled($moduleName) {
      try {
        $moduleHandler = \Drupal::service('module_handler');
        return $moduleHandler->moduleExists($moduleName);
      }
      catch (\Exception $e) {
        $this->logger->error('Error checking module status: @message', [
          '@message' => $e->getMessage()
        ]);
        return FALSE;
      }
    }
    
    protected function hasAnalyzableCode($moduleDir, $moduleName) {
      // Check for source files
      if (is_dir($moduleDir . '/src')) {
        return TRUE;
      }
      
      // Check for module files
      $moduleFiles = [
        "/$moduleName.module",
        "/$moduleName.install",
        "/$moduleName.services.yml",
        "/$moduleName.routing.yml",
      ];
      
      foreach ($moduleFiles as $file) {
        if (file_exists($moduleDir . $file)) {
          return TRUE;
        }
      }
      
      return FALSE;
    }
    
    protected function findCustomModules($path) {
      $modules = [];
      
      try {
        $dir = new \DirectoryIterator($path);
        
        foreach ($dir as $fileinfo) {
          if ($fileinfo->isDir() && !$fileinfo->isDot()) {
            $moduleName = $fileinfo->getBasename();
            $moduleDir = $fileinfo->getPathname();
            
            if (file_exists($moduleDir . '/' . $moduleName . '.info.yml')) {
              if ($moduleName !== 'performance_analyzer' && 
              $this->isModuleInstalled($moduleName) &&
              $this->hasAnalyzableCode($moduleDir, $moduleName)) {
                
                $modules[] = $moduleDir;
                $this->logger->notice('Found active module: @module in @dir', [
                  '@module' => $moduleName,
                  '@dir' => $moduleDir
                ]);
              } else {
                $this->logger->notice('Skipping module @module: Not installed or no analyzable code', [
                  '@module' => $moduleName
                ]);
              }
            }
          }
        }
      }
      catch (\Exception $e) {
        $this->logger->error('Error scanning directory @path: @message', [
          '@path' => $path,
          '@message' => $e->getMessage()
        ]);
        throw $e;
      }
      
      return $modules;
    }
    
    protected function analyzeModule($modulePath) {
      $moduleName = basename($modulePath);
      
      $this->logger->notice('Analyzing module: @module', ['@module' => $moduleName]);
      
      $this->results[$moduleName] = [
        'path' => $modulePath,
        'issues' => [],
        'stats' => [
          'file_count' => 0,
          'service_count' => 0,
          'cache_implementations' => 0,
          'database_queries' => 0,
          'entity_queries' => 0
        ],
        'module_info' => $this->getModuleInfo($moduleName, $modulePath),
        'recommendations' => [],
        'performance_score' => 100
      ];
      
      try {
        // Find all PHP files in the module directory and subdirectories
        $phpFiles = $this->findAnalyzableFiles($modulePath);
        
        if (empty($phpFiles)) {
          $this->logger->notice('No analyzable files found in module: @module', [
            '@module' => $moduleName
          ]);
          unset($this->results[$moduleName]);
          return;
        }
        
        foreach ($phpFiles as $file) {
          $this->results[$moduleName]['stats']['file_count']++;
          $this->analyzeFile($file, $moduleName);
        }
        
        // Calculate final performance score and recommendations
        $this->calculatePerformanceScore($moduleName);
        
        $this->logger->notice('Completed analysis of @module. Found @count PHP files.', [
          '@module' => $moduleName,
          '@count' => count($phpFiles)
        ]);
      }
      catch (\Exception $e) {
        $this->logger->error('Error analyzing module @module: @message', [
          '@module' => $moduleName,
          '@message' => $e->getMessage()
        ]);
        throw $e;
      }
    }
    
    protected function getModuleInfo($moduleName, $modulePath) {
      $infoFile = $modulePath . '/' . $moduleName . '.info.yml';
      if (file_exists($infoFile)) {
        try {
          $info = \Drupal::service('serialization.yaml')->decode(file_get_contents($infoFile));
          return [
            'name' => $info['name'] ?? $moduleName,
            'description' => $info['description'] ?? '',
            'version' => $info['version'] ?? '',
            'core' => $info['core_version_requirement'] ?? '',
            'dependencies' => $info['dependencies'] ?? [],
          ];
        }
        catch (\Exception $e) {
          $this->logger->error('Error reading info file for @module: @message', [
            '@module' => $moduleName,
            '@message' => $e->getMessage()
          ]);
          return [];
        }
      }
      return [];
    }
    
    protected function findAnalyzableFiles($modulePath) {
      $files = [];
      $iterator = new \RecursiveIteratorIterator(
        new \RecursiveDirectoryIterator($modulePath)
      );
      
      foreach ($iterator as $file) {
        if ($file->isFile() && 
        $file->getExtension() === 'php' && 
        !$this->shouldSkipFile($file)) {
          $files[] = $file;
        }
      }
      
      return $files;
    }
    
    protected function shouldSkipFile($file) {
      // Skip test files
      if (strpos($file->getPathname(), '/tests/') !== FALSE) {
        return TRUE;
      }
      
      // Skip files in these directories
      $skipDirs = ['vendor', 'node_modules', 'examples'];
      foreach ($skipDirs as $dir) {
        if (strpos($file->getPathname(), "/$dir/") !== FALSE) {
          return TRUE;
        }
      }
      
      return FALSE;
    }
    
    protected function analyzeFile(\SplFileInfo $file, $moduleName) {
      $relativePath = str_replace(\Drupal::root(), '', $file->getPathname());
      
      try {
        if (!$file->isReadable()) {
          throw new \RuntimeException("File is not readable: {$file->getPathname()}");
        }
        
        $content = file_get_contents($file->getPathname());
        
        if ($content === false) {
          throw new \RuntimeException("Failed to read file contents: {$file->getPathname()}");
        }
        
        $this->analyzeEntityLoading($content, $moduleName, $relativePath);
        $this->analyzeCaching($content, $moduleName, $relativePath);
        $this->analyzeQueries($content, $moduleName, $relativePath);
        $this->analyzeRenderArrays($content, $moduleName, $relativePath);
        $this->analyzeServiceUsage($content, $moduleName, $relativePath);
      }
      catch (\Exception $e) {
        $this->logger->error('Error analyzing file @file: @message', [
          '@file' => $relativePath,
          '@message' => $e->getMessage()
        ]);
        throw $e;
      }
    }
    
    protected function analyzeEntityLoading($content, $moduleName, $file) {
      $patterns = [
        'direct_entity_load' => '/\b(entity|node|taxonomy_term|user)_load\b/',
        'entity_load_multiple' => '/\b(entity|node|taxonomy_term|user)_load_multiple\b/',
        'entity_load_loop' => '/foreach.*{.*?(entity|node|taxonomy_term|user)_load/'
      ];
      
      foreach ($patterns as $type => $pattern) {
        if (preg_match_all($pattern, $content, $matches)) {
          $severity = match ($type) {
            'direct_entity_load', 
            'entity_load_loop' => 'high',
            'entity_load_multiple' => 'medium',
            default => 'low'
          };
          $this->addIssue($moduleName, [
            'type' => 'entity_loading',
            'subtype' => $type,
            'file' => $file,
            'severity' => $severity,
            'message' => $this->getEntityLoadingMessage($type),
            'count' => count($matches[0]),
            'lines' => $this->findMatchingLines($content, $matches[0])
          ]);
        }
      }
    }
    
    protected function analyzeCaching($content, $moduleName, $file) {
      if (strpos($content, 'database') !== false && !strpos($content, 'cache')) {
        $severity = 'high';
        $this->addIssue($moduleName, [
          'type' => 'caching',
          'subtype' => 'missing_cache',
          'file' => $file,
          'severity' => $severity,
          'message' => 'Database operations without caching detected',
          'lines' => []
        ]);
      }
      
      $patterns = [
        'empty_cache' => '/#cache\s*=>\s*\[\s*\]/',
        'missing_tags' => '/->set\([^)]+\).*?(?!->tagCache)/',
        'static_cache' => '/static\s+\$[a-zA-Z_]+/'
      ];
      
      foreach ($patterns as $type => $pattern) {
        if (preg_match_all($pattern, $content, $matches)) {
          $severity = match ($type) {
            'static_cache' => 'high',
            'empty_cache' => 'medium',
            'missing_tags' => 'low'
          };
          $this->addIssue($moduleName, [
            'type' => 'caching',
            'subtype' => $type,
            'file' => $file,
            'severity' => $severity,
            'message' => $this->getCachingMessage($type),
            'count' => count($matches[0]),
            'lines' => $this->findMatchingLines($content, $matches[0])
          ]);
        }
      }
    }
    
    protected function analyzeQueries($content, $moduleName, $file) {
      $patterns = [
        'direct_query' => '/db_query\(/',
        'missing_tag' => '/->execute\(/',
        'query_loop' => '/foreach.*{.*?->execute\(/',
      ];
      
      foreach ($patterns as $type => $pattern) {
        if (preg_match_all($pattern, $content, $matches)) {
          $severity = match ($type) {
            'direct_query', 'query_loop' => 'high',
            'missing_tag' => 'medium',
            default => 'low'
          };
          $this->addIssue($moduleName, [
            'type' => 'query',
            'subtype' => $type,
            'file' => $file,
            'severity' => $severity,
            'message' => $this->getQueryMessage($type),
            'count' => count($matches[0]),
            'lines' => $this->findMatchingLines($content, $matches[0])
          ]);
        }
      }
    }
    
    protected function analyzeRenderArrays($content, $moduleName, $file) {
      $patterns = [
        'missing_cache_metadata' => '/return\s+\[([^\]]*#type[^\]]*)\]/',
        'direct_markup' => '/#markup\s*=>\s*/',
        'inline_css' => '/#attached.*css/',
        'inline_js' => '/#attached.*js/'
      ];
      
      foreach ($patterns as $type => $pattern) {
        if (preg_match_all($pattern, $content, $matches)) {
          $severity = match ($type) {
            'missing_cache_metadata' => 'medium',
            default => 'low'
          };
          $this->addIssue($moduleName, [
            'type' => 'render_array',
            'subtype' => $type,
            'file' => $file,
            'severity' => $severity,
            'message' => $this->getRenderArrayMessage($type),
            'count' => count($matches[0]),
            'lines' => $this->findMatchingLines($content, $matches[0])
          ]);
        }
      }
    }
    
    protected function analyzeServiceUsage($content, $moduleName, $file) {
      $patterns = [
        'container_get' => '/\$container->get\(/',
        'drupal_service' => '/\\\\Drupal::service\(/',
        'static_service' => '/\\\\Drupal::[a-zA-Z_]+\(/',
        'global_container' => '/\\\\Drupal::getContainer\(\)/',
      ];
      
      foreach ($patterns as $type => $pattern) {
        if (preg_match_all($pattern, $content, $matches)) {
          $severity = 'high';  // All service usage issues are high severity
          $this->addIssue($moduleName, [
            'type' => 'service_usage',
            'subtype' => $type,
            'file' => $file,
            'severity' => $severity,
            'message' => $this->getServiceUsageMessage($type),
            'count' => count($matches[0]),
            'lines' => $this->findMatchingLines($content, $matches[0])
          ]);
        }
      }
    }
    
    protected function addIssue($moduleName, $issue) {
      // Apply penalties first
      $penalties = [
        'high' => 25,
        'medium' => 15,
        'low' => 5
      ];
      
      $this->results[$moduleName]['performance_score'] -= $penalties[$issue['severity']] ?? 0;
      $this->results[$moduleName]['performance_score'] = max(0, $this->results[$moduleName]['performance_score']);
      
      // Add issue with original severity
      $this->results[$moduleName]['issues'][] = $issue;
    }
    
    protected function getEntityLoadingMessage($type) {
      $messages = [
        'direct_entity_load' => 'Direct entity loading detected. Use dependency injection and EntityTypeManager service.',
        'entity_load_multiple' => 'Direct entity_load_multiple detected. Use EntityTypeManager service.',
        'entity_load_loop' => 'Entity loading inside a loop detected. Use entity_load_multiple instead.'
      ];
      return $messages[$type] ?? 'Entity loading issue detected';
    }
    
    protected function getCachingMessage($type) {
      $messages = [
        'empty_cache' => 'Empty cache array detected in render array',
        'missing_tags' => 'Cache implementation missing cache tags',
        'static_cache' => 'Static variable caching detected. Consider using Cache API'
      ];
      return $messages[$type] ?? 'Caching issue detected';
    }
    
    protected function getQueryMessage($type) {
      $messages = [
        'direct_query' => 'Direct database query detected. Use Database API.',
        'missing_tag' => 'Database query missing query tag',
        'query_loop' => 'Database query inside loop detected. Consider using IN condition'
      ];
      return $messages[$type] ?? 'Query issue detected';
    }
    
    protected function getRenderArrayMessage($type) {
      $messages = [
        'missing_cache_metadata' => 'Render array missing cache metadata',
        'direct_markup' => 'Direct #markup usage detected. Consider using templates',
        'inline_css' => 'Inline CSS detected. Use library system',
        'inline_js' => 'Inline JavaScript detected. Use library system'
      ];
      return $messages[$type] ?? 'Render array issue detected';
    }
    
    protected function getServiceUsageMessage($type) {
      $messages = [
        'container_get' => 'Direct container access detected. Use dependency injection instead.',
        'drupal_service' => 'Static service call detected. Use dependency injection instead.',
        'static_service' => 'Static service access detected. Use dependency injection instead.',
        'global_container' => 'Global container access detected. Use dependency injection instead.'
      ];
      return $messages[$type] ?? 'Service usage issue detected';
    }
    
    protected function findMatchingLines($content, $matches) {
      $lines = [];
      $contentLines = explode("\n", $content);
      foreach ($matches as $match) {
        foreach ($contentLines as $number => $line) {
          if (strpos($line, $match) !== false) {
            $lines[] = $number + 1;
          }
        }
      }
      return array_unique($lines);
    }
    
    protected function calculatePerformanceScore($moduleName) {
      if (!isset($this->results[$moduleName])) {
        return;
      }
      
      $score = $this->results[$moduleName]['performance_score'];
      
      // Add recommendations based on issues
      if ($score < 70) {
        $this->results[$moduleName]['recommendations'][] = 'Critical performance issues detected. Immediate attention required.';
        $this->results[$moduleName]['recommendations'][] = $this->generateCriticalRecommendations($moduleName);
      }
      elseif ($score < 85) {
        $this->results[$moduleName]['recommendations'][] = 'Moderate performance issues detected. Consider refactoring.';
        $this->results[$moduleName]['recommendations'][] = $this->generateModerateRecommendations($moduleName);
      }
      else {
        $this->results[$moduleName]['recommendations'][] = 'Module performs well. Minor optimizations possible.';
        $this->results[$moduleName]['recommendations'][] = $this->generateMinorRecommendations($moduleName);
      }
    }
    
    protected function generateCriticalRecommendations($moduleName) {
      $recommendations = [];
      $issues = $this->results[$moduleName]['issues'];
      
      foreach ($issues as $issue) {
        if ($issue['severity'] === 'high') {
          switch ($issue['type']) {
            case 'entity_loading':
              $recommendations[] = [
                'title' => 'Entity Loading Optimization',
                'description' => 'Replace direct entity loading with EntityTypeManager service'
              ];
              break;
              case 'caching':
                $recommendations[] = [
                  'title' => 'Caching Implementation',
                  'description' => 'Implement proper caching strategy'
                ];
                break;
                case 'query':
                  $recommendations[] = [
                    'title' => 'Query Optimization',
                    'description' => 'Optimize database queries'
                  ];
                  break;
                }
              }
            }
            return $recommendations;
          }
          
          protected function generateModerateRecommendations($moduleName) {
            $recommendations = [];
            $issues = $this->results[$moduleName]['issues'];
            
            foreach ($issues as $issue) {
              if ($issue['severity'] === 'medium') {
                switch ($issue['type']) {
                  case 'render_array':
                    $recommendations[] = [
                      'title' => 'Render Array Optimization',
                      'description' => 'Improve render array implementation',
                    ];
                    break;
                    case 'static_cache':
                      $recommendations[] = [
                        'title' => 'Static Cache Replacement',
                        'description' => 'Replace static caching with Cache API',
                      ];
                      break;
                    }
                  }
                }
                
                return $recommendations;
              }
              
              protected function generateMinorRecommendations($moduleName) {
                $recommendations = [];
                $issues = $this->results[$moduleName]['issues'];
                
                foreach ($issues as $issue) {
                  if ($issue['severity'] === 'low') {
                    switch ($issue['type']) {
                      case 'code_style':
                        $recommendations[] = [
                          'title' => 'Code Style Optimization',
                          'description' => 'Minor code style improvements',
                        ];
                        break;
                      }
                    }
                  }
                  
                  return $recommendations;
                }
                
                protected function getIssueSeverity($score) {
                  if ($score <= 50) {
                    return 'high';
                  } elseif ($score <= 85) {
                    return 'medium';
                  }
                  return 'low';
                }
              }