<?php

namespace Drupal\performance_analyzer\Service;

use Drupal\Core\File\FileSystemInterface;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\Component\Serialization\Json;
use Drupal\Core\Render\RendererInterface;
use Symfony\Component\HttpFoundation\BinaryFileResponse;
use Symfony\Component\HttpFoundation\ResponseHeaderBag;

class ReportGeneratorService {
    protected $analyzer;
    protected $fileSystem;
    protected $time;
    protected $serializer;
    protected $renderer;

    public function __construct(
        PerformanceAnalyzerService $analyzer,
        FileSystemInterface $fileSystem,
        TimeInterface $time,
        Json $serializer,
        RendererInterface $renderer
    ) {
        $this->analyzer = $analyzer;
        $this->fileSystem = $fileSystem;
        $this->time = $time;
        $this->serializer = $serializer;
        $this->renderer = $renderer;
    }

    public function generateReport($format = 'html') {
        $report = $this->analyzer->analyze();
        
        switch ($format) {
            case 'json':
                return $this->generateJsonReport($report);
            case 'html':
                return $this->generateHtmlReport($report);
            default:
                throw new \InvalidArgumentException("Unsupported format: $format");
        }
    }

    protected function generateJsonReport($report) {
        $timestamp = $this->time->getCurrentTime();
        $filename = "performance_report_{$timestamp}.json";
        $filepath = 'temporary://' . $filename;
        
        $data = $this->serializer->encode($report);
        $this->fileSystem->saveData($data, $filepath, FileSystemInterface::EXISTS_REPLACE);
        
        return new BinaryFileResponse($this->fileSystem->realpath($filepath), 200, [
            'Content-Type' => 'application/json',
            'Content-Disposition' => ResponseHeaderBag::DISPOSITION_ATTACHMENT . '; filename="' . $filename . '"',
        ]);
    }

    protected function generateHtmlReport($report) {
        $build = [
            '#theme' => 'performance_analyzer_report',
            '#report' => $report,
            '#attached' => [
                'library' => ['performance_analyzer/report'],
            ],
        ];

        $html = $this->renderer->renderRoot($build);
        
        $timestamp = $this->time->getCurrentTime();
        $filename = "performance_report_{$timestamp}.html";
        $filepath = 'temporary://' . $filename;
        
        $this->fileSystem->saveData($html, $filepath, FileSystemInterface::EXISTS_REPLACE);
        
        return new BinaryFileResponse($this->fileSystem->realpath($filepath), 200, [
            'Content-Type' => 'text/html',
            'Content-Disposition' => ResponseHeaderBag::DISPOSITION_ATTACHMENT . '; filename="' . $filename . '"',
        ]);
    }

    public function generateDownloadUrl($format) {
        return \Drupal\Core\Url::fromRoute('performance_analyzer.download_report', [
            'format' => $format,
        ]);
    }
}